#include <iostream>  
using namespace std;

#define MAX 100

struct phone_num_t {
	char vanity[MAX];
	char normalized[MAX];
};

const char phone_digits[10][5] = {"+", "", "ABC", "DEF", "GHI", "JKL", "MNO", "PQRS", "TUV", "WXYZ"};

const char other_valid_chars[] = {'+', '-', ' ', '(', ')','[', ']','.','\0'};


void display_phone_num(phone_num_t p)
{ // Display the phone number to the console.
	cout << "Number = " << p.vanity << "\t<" << p.normalized << ">" << endl;	
}

bool is_alpha_or_num(char c)
{
	if (('a' <= c) && (c <= 'z'))
		return true;
	if (('A' <= c) && (c <= 'Z'))
		return true;
	if (('0' <= c) && (c <= '9'))
		return true;
	return false;
}

bool is_other_valid_ph_char(char c)
{
	int i = 0;	
	while (other_valid_chars[i] != '\0')
	{
		if (c == other_valid_chars[i]) 
			return true;
		i++;
	}
	return false;
}

bool is_char_in_str(char needle, const char str[])
{
	int i = 0;	
	while (str[i] != '\0')
	{
		if (needle == str[i]) return true;
		i++;
	}
	return false;
}

char toupper(char c)
{
	/* A useful check is to ensure that our function has some well defined behavior on any input c; in this case, we only change the value if it is actually a lowercase letter. */
	if ((c >= 'a') && (c <= 'z'))
		return (c - 'a') + 'A';
	return c;
}


char convert_alpha_to_ph_num(char c)
{
	//cout << "convert_alpha_to_ph_num('" << c << "') " << endl;
	for (int i =0; i <= 9; i++)
	{
		/* Rationale :
		if (c is in phone_digits[i])
			return i;
		*/
		if (is_char_in_str(toupper(c), phone_digits[i]))
		{
			//cout << c << " maps to " << i << endl;
			// reminder: i is an int, we want a digit from '0' - '9'
		 	return '0' + i;	
		}

	}	
	return c;
}

void set_nomalization(phone_num_t &p)
{
	//cout << p.vanity << endl;

	int i = 0;
	int j = 0;
	while (p.vanity[i] != '\0')
	{
		if (is_alpha_or_num(p.vanity[i]))
		{ // then we want to copy it over.
			p.normalized[j] = convert_alpha_to_ph_num(p.vanity[i]);
			j++;		
		}
		else if (is_other_valid_ph_char(p.vanity[i]))
		{
			// Do nothing
		}
		else
		{
			cerr << "FYI: I don't know what `" << p.vanity[i] << "' is, so I'm dropping it" << endl;
		}
		i++;
	}
	p.normalized[j] = '\0';
	
	//cout << p.normalized << endl;
}

bool are_two_phones_same(phone_num_t a, phone_num_t b)
{
	// a.normalized
	// b.normalized
	int i = 0;
	while (a.normalized[i] != '\0')
	{
		if (a.normalized[i] != b.normalized[i])
			return false;
		i++;
	}
	// Reached the end of a.normalized
	
	// b.normalized is too long
	if (b.normalized[i] != '\0') return false;

	return true;
}

int main()
{
	/* Did some testing of my is_char_in_str function here:
	char test_str[] = "hello world";
	char t = 'd';
	if (is_char_in_str(t, test_str)	)
		cout << t << " is in " << test_str << endl;
	return 0;
	*/

	phone_num_t one, second;

	//cout << "How big a phone_num_t? = "<< sizeof(one) << endl;
	// The struct is a block of sequential bytes  

	// one.vanity This is how we refer to a field
	cout << "Give me a first number please: ";
	cin.getline(one.vanity, MAX);
	// Here's how we might ensure that the normalized field is initialized
	// 	one.normalized[0] = '\0'; 
	// instead we'll set it to the normalized version of the vanity.
	set_nomalization(one);
	display_phone_num(one);

	cout << "Give me a second number please: ";
	cin.getline(second.vanity, MAX);
	set_nomalization(second);
	display_phone_num(second);

	if (are_two_phones_same(one, second))
		cout << "They're the same number!" << endl;
	else
		cout << "They're NOT the same number!" << endl;

	
	/* 
	 *  Note: to read in a string from the console, we provide a char[] and a max length:
		cin.getline( <char[]>, <len> );	

		This is different from the version with C++ strings:
		cin.getline( <string> );	
	 */
	
	return 0;
}


